@@ -49,6 +49,7 @@ if 'f' in globals(): del f
 
 from clock import *
 from connections import *
+from synapses import *
 from directcontrol import *
 from stateupdater import *
 from monitor import *
@@ -70,10 +71,11 @@ from magic import *
 from stdp import *
 from stp import *
 from timedarray import *
+from deprecated.multiplespikegeneratorgroup import *
 from tests.simpletest import *
 
-__version__ = '1.3.1'
-__release_date__ = '2011-12-16'
+__version__ = '1.4.1'
+__release_date__ = '2013-04-30'
 
 ### Define global preferences which are not defined anywhere else
 
@@ -105,6 +107,18 @@ define_global_preference(
          ''')
 set_global_preferences(gcc_options=['-ffast-math'])
 
+define_global_preference(
+    'openmp', 'False',
+    desc='''
+        Whether or not to use OpenMP pragmas in generated C code. If supported
+        on your compiler (gcc 4.2+) it will use multiple CPUs and can run
+        substantially faster. However, if you are already running several
+        simulations in parallel this will not improve the speed and may even
+        slow it down. In addition, for smaller networks or for simpler neuron
+        models the parallelisation overheads can make it take longer.
+        ''')
+set_global_preferences(openmp=False)
+
 define_global_preference(
     'usecodegen', 'False',
     desc='''
