@@ -5,6 +5,8 @@ from constructionmatrix import *
 from connectionmatrix import *
 from construction import *
 from propagation_c_code import *
+from scipy.sparse import issparse
+import gc
 # we do this at the bottom because of order of import issues
 #from delayconnection import * 
 
@@ -82,7 +84,8 @@ class Connection(magic.InstanceTracker, ObjectContainer):
         recognised by NumPy). Note that due to internal implementation details,
         passing a full matrix rather than a sparse one may slow down your code
         (because zeros will be propagated as well as nonzero values).
-        **WARNING:** No unit checking is done at the moment.
+        **WARNING:** No unit checking is done at the moment. For a more
+        efficient low-level method see :meth:`connect_from_sparse`.
 
     Additionally, you can directly access the matrix of weights by writing::
     
@@ -149,6 +152,10 @@ class Connection(magic.InstanceTracker, ObjectContainer):
         have in advance, specify the ``nnzmax`` keyword to set the
         initial size of the array. 
     
+    **Low level methods**
+    
+    .. automethod:: connect_from_sparse
+    
     **Advanced information**
     
     The following methods are also defined and used internally, if you are
@@ -175,6 +182,7 @@ class Connection(magic.InstanceTracker, ObjectContainer):
             return
         self.source = source # pointer to source group
         self.target = target # pointer to target group
+        self.state = state
         if isinstance(state, str): # named state variable
             self.nstate = target.get_var_index(state)
         else:
@@ -206,7 +214,7 @@ class Connection(magic.InstanceTracker, ObjectContainer):
                 weight = 1.0
             if sparseness is None:
                 sparseness = 1
-            if isinstance(weight, sparse.lil_matrix) or isinstance(weight, ndarray):
+            if issparse(weight) or isinstance(weight, ndarray):
                 self.connect(W=weight)
             elif sparseness == 1:
                 self.connect_full(weight=weight)
@@ -276,6 +284,7 @@ class Connection(magic.InstanceTracker, ObjectContainer):
         if not self.iscompressed:
             self.W = self.W.connection_matrix()
             self.iscompressed = True
+            gc.collect() # collect after changing matrix type to free up mem
 
     def reinit(self):
         '''
@@ -399,6 +408,32 @@ class Connection(magic.InstanceTracker, ObjectContainer):
             raise AttributeError, 'The connected (sub)groups must have the same size.'
         # TODO: unit checking
         self.connect(P, Q, float(weight) * eye_lil_matrix(len(P)))
+        
+    def connect_from_sparse(self, W, delay=None, column_access=True):
+        '''
+        Bypasses the usual Brian mechanisms for constructing and compressing
+        matrices, and allows you to directly set the weight (and optionally
+        delay) matrices from scipy sparse matrix objects. This can be
+        more memory efficient, because the default sparse matrix used for
+        construction is ``scipy.sparse.lil_matrix`` which is very memory
+        inefficient.
+        
+        Arguments:
+        
+        ``W``
+            The weight matrix
+        ``delay``
+            Optional delay matrix if you are using heterogeneous delays.
+        ``column_access=True``
+            Set to ``False`` to save memory if you are not using STDP.
+        
+        .. warning::
+            This is a low-level method that bypasses the usual checks on the
+            data. In particular, make sure that if you pass a weight and
+            delay matrix, that they have the same structure.
+        '''
+        set_connection_from_sparse(self, W, delay=delay,
+                                   column_access=column_access)
 
     def __getitem__(self, i):
         return self.W.__getitem__(i)
@@ -407,4 +442,16 @@ class Connection(magic.InstanceTracker, ObjectContainer):
         # TODO: unit checking
         self.W.__setitem__(i, x)
 
+    def __repr__(self):
+        return '%s(%s, %s, state=%s)' % (self.__class__.__name__, 
+                                         repr(self.source),
+                                         repr(self.target),
+                                         repr(self.nstate))
+    def __str__(self):
+        return '%s from %s to %s, acting on state variable %d' % (self.__class__.__name__,
+                                                                  str(self.source),
+                                                                  str(self.target),
+                                                                  self.nstate) 
+
+
 from delayconnection import *
