@@ -262,7 +262,6 @@ class Sound(BaseSound, numpy.ndarray):
 
         sliceattr = [v for v in [key.start, key.stop] if v is not None]
         slicedims = array([units.have_same_dimensions(flag, second) for flag in sliceattr])
-
         attrisint = array([isinstance(v, int) for v in sliceattr])
         s = sum(attrisint)
         if s!=0 and s!=len(sliceattr):
@@ -304,24 +303,25 @@ class Sound(BaseSound, numpy.ndarray):
             channel=key[1]
             key=key[0]
         
-        if isinstance(key,int) or isinstance(key,float):
+        if isinstance(key,int):
             return np.ndarray.__setitem__(self,(key,channel),value)
-        if isinstance(key,Quantity):
+        if isinstance(key,float):
             return np.ndarray.__setitem__(self,(int(rint(key*self.samplerate)),channel),value)
 
         sliceattr = [v for v in [key.start, key.step, key.stop] if v is not None]
-        slicedims=array([units.have_same_dimensions(flag,second) for flag in sliceattr])
-        if not slicedims.any():
+        slicedims = array([units.have_same_dimensions(flag, second) for flag in sliceattr])
+        attrisint = array([isinstance(v, int) for v in sliceattr])
+        s = sum(attrisint)
+        if s!=0 and s!=len(sliceattr):
+            raise ValueError('Slice attributes must be all ints or all times')
+        if s==len(sliceattr): # all ints
             # If value is a mono sound its shape will be (N, 1) but the numpy
             # setitem will have shape (N,) so in this case it's a shape mismatch
             # so we squeeze the array to make sure this doesn't happen.
             if isinstance(value,Sound) and channel!=slice(None):
                 value=value.squeeze()
-            return asarray(self).__setitem__((key,channel),value)
+            return asarray(self).__setitem__((key,channel),value) # returns None
 
-#            print np.ndarray.__getitem__(self, (key, channel)).shape
-#            print key, channel
-#            return np.ndarray.__setitem__(self, (key, channel), value)
         if not slicedims.all():
             raise DimensionMismatchError('Slicing',*[units.get_unit(d) for d in sliceattr])
 
@@ -334,7 +334,6 @@ class Sound(BaseSound, numpy.ndarray):
         if (start is not None and start<0*ms) or stop > self.duration:
             raise IndexError('Slice bigger than Sound object')
         if start is not None: start = int(rint(start*self.samplerate))
-        #if (stop is not None) and (not stop=self.duration): stop = int(stop*self.samplerate)
         stop = int(rint(stop*self.samplerate))
         return self.__setitem__((slice(start,stop),channel),value)
 
@@ -385,7 +384,7 @@ class Sound(BaseSound, numpy.ndarray):
         else:
             if self.nchannels>1:
                 sounds = [self.channel(i).shifted(duration, fractional=True, filter_length=filter_length) for i in xrange(self.nchannels)]
-                return Sound(array(sounds), samplerate=self.samplerate)
+                return Sound(hstack(sounds), samplerate=self.samplerate)
             # Adapted from
             # http://www.labbookpages.co.uk/audio/beamforming/fractionalDelay.html
             delay = duration*self.samplerate
@@ -435,10 +434,11 @@ class Sound(BaseSound, numpy.ndarray):
 
     def _init_mixer(self):
         global _mixer_status
-        if _mixer_status==[-1,-1] or _mixer_status[0]!=self.nchannels or _mixer_status != self.samplerate:
+        if _mixer_status==[-1,-1] or _mixer_status[0]!= self.nchannels or _mixer_status[1] != self.samplerate:
             pygame.mixer.quit()
             pygame.mixer.init(int(self.samplerate), -16, self.nchannels)
             _mixer_status=[self.nchannels,self.samplerate]
+
     
     def play(self, normalise=False, sleep=False):
         '''
@@ -506,13 +506,16 @@ class Sound(BaseSound, numpy.ndarray):
             Z = 10 * log10(Z)
         Z = flipud(Z)
         if restricted:
-            imshow(Z, extent=(0, amax(bins), freqs[I2[0]], freqs[I2[-1]]), aspect='auto')
+            imshow(Z, extent=(0, amax(bins), freqs[I2[0]], freqs[I2[-1]]),
+                   origin='upper', aspect='auto')
         else:
-            imshow(Z, extent=(0, amax(bins), freqs[0], freqs[-1]), aspect='auto')
+            imshow(Z, extent=(0, amax(bins), freqs[0], freqs[-1]),
+                   origin='upper', aspect='auto')
         xlabel('Time (s)')
         ylabel('Frequency (Hz)')
         return (pxx, freqs, bins, im)
 
+    @check_units(low=Hz, high=Hz)
     def spectrum(self, low=None, high=None, log_power=True, display=False):
         '''
         Returns the spectrum of the sound and optionally plots it.
@@ -533,7 +536,9 @@ class Sound(BaseSound, numpy.ndarray):
         '''
         if self.nchannels>1:
             raise ValueError('Can only plot spectrum for mono sounds.')
-        sp = numpy.fft.fft(array(self))
+
+        # Flatten array, fft operates on the last axis by default
+        sp = numpy.fft.fft(array(self).flatten())
         freqs = array(range(len(sp)), dtype=float64) / len(sp) * float64(self.samplerate)
         pxx = abs(sp) ** 2
         phase = unwrap(mod(angle(sp), 2 * pi))
@@ -683,6 +688,7 @@ class Sound(BaseSound, numpy.ndarray):
             target[target.nsamples-sz:, :] *= multiplier[::-1]
         return target
     
+    @check_units(duration=second)
     def ramped(self, when='onset', duration=10*ms, envelope=None):
         '''
         Returns a ramped version of the sound (see :meth:`Sound.ramp`).
@@ -703,6 +709,7 @@ class Sound(BaseSound, numpy.ndarray):
         
     
     @staticmethod
+    @check_units(frequency=Hz, duration=second, samplerate=Hz)
     def tone(frequency, duration, phase=0, samplerate=None, nchannels=1):
         '''
         Returns a pure tone at frequency for duration, using the default
@@ -721,7 +728,7 @@ class Sound(BaseSound, numpy.ndarray):
         if phase.size>nchannels and nchannels==1:
             nchannels = phase.size
         if frequency.size==nchannels:
-            frequency.shape = (nchannels, 1)
+            frequency.shape = (1, nchannels)
         if phase.size==nchannels:
             phase.shape =(nchannels, 1)
         t = arange(0, duration, 1)/samplerate
@@ -730,6 +737,7 @@ class Sound(BaseSound, numpy.ndarray):
         return Sound(x, samplerate)
 
     @staticmethod
+    @check_units(f0=Hz, duration=second, samplerate=Hz)
     def harmoniccomplex(f0, duration, amplitude=1, phase=0, samplerate=None, nchannels=1):
         '''
         Returns a harmonic complex composed of pure tones at integer multiples
@@ -737,7 +745,7 @@ class Sound(BaseSound, numpy.ndarray):
         The ``amplitude`` and
         ``phase`` keywords can be set to either a single value or an
         array of values. In the former case the value is set for all
-        harmonics, and harmonics up the the sampling frequency are
+        harmonics, and harmonics up to the sampling frequency are
         generated. In the latter each harmonic parameter is set
         separately, and the number of harmonics generated corresponds
         to the length of the array.
@@ -767,6 +775,7 @@ class Sound(BaseSound, numpy.ndarray):
         return Sound(x,samplerate)
     
     @staticmethod
+    @check_units(duration=second, samplerate=Hz)
     def whitenoise(duration, samplerate=None, nchannels=1):
         '''
         Returns a white noise. If the samplerate is not specified, the global
@@ -778,6 +787,7 @@ class Sound(BaseSound, numpy.ndarray):
         return Sound(x, samplerate)
 
     @staticmethod
+    @check_units(duration=second, samplerate=Hz)
     def powerlawnoise(duration, alpha, samplerate=None, nchannels=1,normalise=False):
         '''
         Returns a power-law noise for the given duration. Spectral density per unit of bandwidth scales as 1/(f**alpha).
@@ -839,26 +849,33 @@ class Sound(BaseSound, numpy.ndarray):
     
             
     @staticmethod
+    @check_units(duration=second, samplerate=Hz)
     def pinknoise(duration, samplerate=None, nchannels=1, normalise=False):
         '''
         Returns pink noise, i.e :func:`powerlawnoise` with alpha=1
         '''
-        return Sound.powerlawnoise(duration,1.0,samplerate=samplerate,normalise=False)
+        return Sound.powerlawnoise(duration, 1.0, samplerate=samplerate,
+                                   nchannels=nchannels, normalise=normalise)
     
     @staticmethod
-    def brownnoise(duration, samplerate=None, nchannels=1,normalise=False):
+    @check_units(duration=second, samplerate=Hz)
+    def brownnoise(duration, samplerate=None, nchannels=1, normalise=False):
         '''
         Returns brown noise, i.e :func:`powerlawnoise` with alpha=2
         '''
-        return Sound.powerlawnoise(duration,2.0,samplerate=samplerate,normalise=False)
+        return Sound.powerlawnoise(duration, 2.0, samplerate=samplerate,
+                                   nchannels=nchannels, normalise=normalise)
     
     @staticmethod
+    @check_units(duration=second, samplerate=Hz)
     def irns(delay, gain, niter, duration, samplerate=None, nchannels=1):
         '''
         Returns an IRN_S noise. The iterated ripple noise is obtained trough
         a cascade of gain and delay filtering. 
         For more details: see Yost 1996 or chapter 15 in Hartman Sound Signal Sensation.
         '''
+        if nchannels!=1:
+            raise ValueError("nchannels!=1 not supported.")
         samplerate = get_samplerate(samplerate)
         noise=Sound.whitenoise(duration)
         splrate=noise.samplerate
@@ -875,6 +892,7 @@ class Sound(BaseSound, numpy.ndarray):
         return Sound(x,samplerate)
     
     @staticmethod
+    @check_units(duration=second, samplerate=Hz)
     def irno(delay, gain, niter, duration, samplerate=None, nchannels=1):
         '''
         Returns an IRN_O noise. The iterated ripple noise is obtained many attenuated and
@@ -896,6 +914,7 @@ class Sound(BaseSound, numpy.ndarray):
         return Sound(x, samplerate)
 
     @staticmethod
+    @check_units(duration=second, samplerate=Hz)
     def click(duration, peak=None, samplerate=None, nchannels=1):
         '''
         Returns a click of the given duration.
@@ -916,6 +935,7 @@ class Sound(BaseSound, numpy.ndarray):
         return Sound(x, samplerate)
     
     @staticmethod
+    @check_units(duration=second, samplerate=Hz)
     def clicks(duration, n, interval, peak=None, samplerate=None, nchannels=1):
         '''
         Returns a series of n clicks (see :func:`click`) separated by interval.
@@ -924,6 +944,7 @@ class Sound(BaseSound, numpy.ndarray):
         return oneclick[:interval].repeat(n)
 
     @staticmethod
+    @check_units(duration=second, samplerate=Hz)
     def silence(duration, samplerate=None, nchannels=1):
         '''
         Returns a silent, zero sound for the given duration. Set nchannels to set the number of channels.
@@ -934,6 +955,7 @@ class Sound(BaseSound, numpy.ndarray):
         return Sound(x, samplerate)
 
     @staticmethod
+    @check_units(pitch=Hz, duration=second, samplerate=Hz)
     def vowel(vowel=None, formants=None, pitch=100*Hz, duration=1*second,
           samplerate=None, nchannels=1):
         '''
@@ -1097,6 +1119,16 @@ class Sound(BaseSound, numpy.ndarray):
         
         return Sound(data, samplerate=framerate*Hz)
 
+    def __repr__(self):
+        arrayrep = repr(asarray(self))
+        arrayrep = '\n'.join('    '+l for l in arrayrep.split('\n'))
+        return 'Sound(\n'+arrayrep+',\n    '+repr(self.samplerate)+')'
+    
+    def __str__(self):
+        return 'Sound duration %s, channels %s, samplerate %s' % (self.duration,
+                                                                  self.nchannels,
+                                                                  self.samplerate)
+
     def __reduce__(self):
         return (_load_Sound_from_pickle, (asarray(self), float(self.samplerate)))
 
