@@ -39,14 +39,14 @@ if properly coded.
 '''
 
 __all__ = ['VanRossumMetric','SpikeMonitor', 'PopulationSpikeCounter', 'SpikeCounter', 'FileSpikeMonitor', 'StateMonitor', 'ISIHistogramMonitor', 'Monitor',
-           'PopulationRateMonitor', 'StateSpikeMonitor', 'MultiStateMonitor', 'RecentStateMonitor', 'CoincidenceCounter', 'CoincidenceMatrixCounter', 'StateHistogramMonitor']
+           'PopulationRateMonitor', 'StateSpikeMonitor', 'MultiStateMonitor', 'RecentStateMonitor', 'CoincidenceCounter', 'CoincidenceMatrixCounter', 'StateHistogramMonitor', 'AERSpikeMonitor']
 
 from units import *
 from connections import Connection, SparseConnectionVector
 from numpy import array, zeros, mean, histogram, linspace, tile, digitize,     \
         copy, ones, rint, exp, arange, convolve, argsort, mod, floor, asarray, \
         maximum, Inf, amin, amax, sort, nonzero, setdiff1d, diag, hstack, resize,\
-         inf, var,tril,empty,float64,array,sum
+         inf, var, tril, empty, float64, array, sum, int32, ceil
 from scipy.spatial.distance import sqeuclidean
 from itertools import repeat, izip
 from clock import guess_clock, EventClock, Clock
@@ -60,6 +60,7 @@ from neurongroup import NeuronGroup
 import bisect
 from base import *
 from time import time
+import datetime
 try:
     import pylab, matplotlib
 except:
@@ -95,7 +96,7 @@ class SpikeMonitor(Connection, Monitor):
         numbers that have fired called each step, to define
         custom spike monitoring.
     
-    Has three attributes:
+    Has attributes:
     
     ``nspikes``
         The number of recorded spikes
@@ -106,6 +107,9 @@ class SpikeMonitor(Connection, Monitor):
         A dictionary with keys the indices of the neurons, and values an
         array of the spike times of that neuron. For example,
         ``t=M.spiketimes[3]`` gives the spike times for neuron 3.
+    ``it``
+        Return a tuple ``(i, t)`` where ``i`` and ``t`` are the arrays of spike
+        indices and corresponding spike times (int and float).
 
     For ``M`` a :class:`SpikeMonitor`, you can also write:
     
@@ -131,7 +135,8 @@ class SpikeMonitor(Connection, Monitor):
         source.set_max_delay(delay)
         self.delay = int(delay / source.clock.dt) # Synaptic delay in time bins
         self._newspikes = True
-        if function != None:
+        self.custom_function = function is not None
+        if self.custom_function:
             self.propagate = function
 
     def reinit(self):
@@ -179,6 +184,32 @@ class SpikeMonitor(Connection, Monitor):
                 self._spiketimes[i] = array(self._spiketimes[i])
         return self._spiketimes
     spiketimes = property(fget=getspiketimes)
+    
+    @property
+    def it(self):
+        if len(self.spikes)==0:
+            return array([], dtype=int), array([], dtype=float)
+        i, t = zip(*self.spikes)
+        return array(i, dtype=int), array(t, dtype=float)
+
+    def __repr__(self):
+        repr_str = 'SpikeMonitor(%s, record=%s' % (repr(self.source),
+                                                   repr(self.record)) 
+        if self.delay > 0:
+            repr_str += ', delay=%s' % repr(self.delay * self.source.clock.dt)
+        if self.custom_function:
+            repr_str += ', function_%s' % repr(self.propagate)
+        
+        repr_str += ')'
+        
+        return repr_str 
+
+    def __str__(self):
+        description = "%s monitoring %s" % (self.__class__.__name__, str(self.source))
+        if self.delay > 0:
+            description += ' with a delay of %s' % str(self.delay * self.source.clock.dt)
+        
+        return description
 
 #    def getvspikes(self):
 #        if isinstance(self.source, VectorizedNeuronGroup):
@@ -247,6 +278,14 @@ class PopulationSpikeCounter(SpikeMonitor):
     '''
     def __init__(self, source, delay=0):
         SpikeMonitor.__init__(self, source, record=False, delay=delay)
+        
+    def __repr__(self):
+        repr_str = '%s(%s' % (self.__class__.__name__, repr(self.source))
+        if self.delay > 0:
+            repr_str += ', delay=%s' % repr(self.delay * self.source.clock.dt)
+        repr_str += ')'
+        
+        return repr_str
 
 
 class SpikeCounter(PopulationSpikeCounter):
@@ -343,7 +382,11 @@ class StateSpikeMonitor(SpikeMonitor):
     def propagate(self, spikes):
         if len(spikes):
             self.nspikes += len(spikes)
-            recordedstate = [ [x * u for x in v[spikes]] for v, u in izip(self._vars, self._units) ]
+            recordedstate = [ [ x * u for x in self.source.state_(varname)[spikes]] for varname, u in izip(self._varnames, self._units)]
+            # the above line used to be this one:
+            # recordedstate = [ [x * u for x in v[spikes]] for v, u in izip(self._vars, self._units) ]
+            # which didn't take the values of the variables if they were defined by a static eqn
+            # a = b + c : 1
             self.spikes += zip(spikes, repeat(self.source.clock.t), *recordedstate)
 
     def __getitem__(self, i):
@@ -364,6 +407,13 @@ class StateSpikeMonitor(SpikeMonitor):
         else:
             return array([x[v] for x in self.spikes])
 
+    def __repr__(self):
+        return 'StateSpikeMonitor(%s, var=%s)' % (repr(self.source),
+                                                  repr(self._varnames))
+
+    def __str__(self):
+        return 'StateSpikeMonitor monitoring variable(s) %s of: %s' % (str(self._varnames),
+                                                                      str(self.source))
 
 class HistogramMonitorBase(SpikeMonitor):
     pass
@@ -422,7 +472,24 @@ class ISIHistogramMonitor(HistogramMonitorBase):
             else:
                 self.count[:-1] += h
                 self.count[-1] += len(isi) - sum(h)
+    
+    def __repr__(self):
+        repr_str = "%s(%s, bins=%s" % (self.__class__.__name__,
+                                       repr(self.source), repr(self.bins))
+        if self.delay > 0:
+            repr_str += ', delay=%s' % repr(self.delay * self.source.clock.dt)
+        
+        repr_str += ')'
+        
+        return repr_str
 
+    def __str__(self):
+        description = '%s monitoring %s' % (self.__class__.__name__,
+                                            str(self.source))
+        if self.delay > 0:
+            description += ' with a delay of %s' % str(self.delay * self.source.clock.dt)
+        
+        return description
 
 class FileSpikeMonitor(SpikeMonitor):
     """Records spikes to a file
@@ -441,7 +508,7 @@ class FileSpikeMonitor(SpikeMonitor):
     
     Has one additional method:
     
-    ``close_file()``
+    ``close()``
         Closes the file manually (will happen automatically when
         the program ends).
     """
@@ -460,10 +527,60 @@ class FileSpikeMonitor(SpikeMonitor):
             for i in spikes:
                 self.f.write(str(i) + ", " + str(float(self.source.clock.t)) + "\n")
 
-    def close_file(self):
+    def close(self):
         self.f.close()
 
 
+# The default header for AER files (see AERSpikeMonitor init):
+AER_HEADER = """#!AER-DAT2.0
+# This is a raw AE data file - do not edit
+# Data format is int32 address, int32 timestamp (8 bytes total), repeated for each event
+# Timestamps tick is %s
+# created with the Brian simulator on """
+class AERSpikeMonitor(FileSpikeMonitor):
+    """Records spikes to an AER file
+    
+    Initialised as::
+    
+        FileSpikeMonitor(source, filename[, record=False])
+    
+    Does everything that a :class:`SpikeMonitor` does except ONLY records
+    the spikes to the named file in AER format, during the simulation.
+    These spikes can then
+    be reloaded via the io.load_aer function and used later into a
+    SpikeGeneratorGroup. 
+    
+    It is about three times faster than the :class:`FileSpikeMonitor` and
+    creates  smaller files. On the other hand those files are not
+    human-readable because they are in binary format. 
+
+    
+    Has one additional method:
+    
+    ``close()``
+        Closes the file manually (will happen automatically when
+        the program ends).
+
+    
+    """
+    def __init__(self, source, filename, record=False, delay=0):
+        super(FileSpikeMonitor, self).__init__(source, record, delay)
+        self.filename = filename
+        self.f = open(filename, 'wb')
+        header = AER_HEADER % self.source.clock.dt.__repr__()
+        header += str(datetime.datetime.now()) + '\n'
+        self.f.write(header)
+
+    def propagate(self, spikes):
+#        super(AERSpikeMonitor, self).propagate(spikes)
+        addr = array(spikes, dtype = int32)
+        x = zeros(2*len(addr), dtype = int32)
+        x[1::2] = addr[::-1]
+        x[::2] = int(ceil(float(self.source.clock.t/self.source.clock.dt)))
+        s = x.tostring()[::-1]
+        self.f.write(s)
+
+
 class PopulationRateMonitor(SpikeMonitor):
     '''
     Monitors and stores the (time-varying) population rate
@@ -845,6 +962,26 @@ class StateMonitor(NetworkOperation, Monitor):
             values[i,array(spikemonitor[neuron]/dt, dtype=int)] = value
         #self._values = values # or converted back to a list?
 
+    def __repr__(self):        
+        repr_str = "%s(%s, varname=%s, clock=%s, record=%s" % (self.__class__.__name__,
+                                                               repr(self.P), 
+                                                               repr(self.varname),
+                                                               repr(self.clock),
+                                                               self.record)
+        if self.timestep != 1:
+            repr_str += ', timestep=%d' % self.timestep
+        if self.when != 'end':
+            repr_str += ", when='%s'" % self.when
+        
+        repr_str += ')'
+        
+        return repr_str
+    
+    def __str__(self):
+        return "%s monitoring variable '%s' of: %s" % (self.__class__.__name__,
+                                                      str(self.varname),
+                                                      str(self.P)) 
+
 
 class RecentStateMonitor(StateMonitor):
     '''
@@ -1083,7 +1220,7 @@ class MultiStateMonitor(NetworkOperation):
         pass
 
 
-class CoincidenceCounter(SpikeMonitor):
+class CoincidenceCounter(SpikeCounter):
     """
     Coincidence counter class.
     
@@ -1145,7 +1282,7 @@ class CoincidenceCounter(SpikeMonitor):
     ``target_length``
         The number of spikes in the target spike train associated to each neuron.
     """
-    def __init__(self, source, data=None, spiketimes_offset=None, spikedelays=None,
+    def __init__(self, source, data, spiketimes_offset=None, spikedelays=None,
                  coincidence_count_algorithm='exclusive', onset=None, delta=4 * ms):
 
         source.set_max_delay(0)
