@@ -31,31 +31,30 @@
 # The fact that you are presently reading this means that you have had
 # knowledge of the CeCILL license and that you accept its terms.
 # ----------------------------------------------------------------------------------
-# 
+#
+import copy
+import gc
+import magic
+import time
+from collections import defaultdict
+from itertools import chain
+from operator import isSequenceType
+from inspect import *
+ 
+from brian.base import *
+from brian.clock import guess_clock, Clock
+from brian.connections import *
+from brian.globalprefs import *
+from brian.neurongroup import NeuronGroup
+from brian.units import second
+from brian.utils.progressreporting import *
+
 '''
 Network class
 '''
 __all__ = ['Network', 'MagicNetwork', 'NetworkOperation', 'network_operation', 'run',
            'reinit', 'stop', 'clear', 'forget', 'recall']
 
-from Queue import Queue
-from connections import *
-from neurongroup import NeuronGroup
-from clock import guess_clock, Clock
-import magic
-from inspect import *
-from operator import isSequenceType
-import types
-from itertools import chain
-from collections import defaultdict
-import copy
-from base import *
-from units import second
-import time
-from utils.progressreporting import *
-from globalprefs import *
-import gc
-import heapq
 
 globally_stopped = False
 
@@ -95,6 +94,8 @@ class Network(object):
     ``add(...)``
         Add additional objects after initialisation, works the same way
         as initialisation.
+    ``remove(...)``
+        Remove objects from the Network.
     ``run(duration[, report[, report_period]])``
         Runs the network for the given duration. See below for details about
         what happens when you do this. See documentation for :func:`run` for
@@ -201,14 +202,31 @@ class Network(object):
         self._all_operations = []
         self.update_schedule_standard()
         self.prepared = False
+        self._added_objects = []
         for o in chain(args, kwds.itervalues()):
             self.add(o)
 
+    def unprepare(self):
+        if self.prepared:
+            # _added_objects keeps track of everything the user has added in
+            # order, and can be used when re-preparing a Network, but the way
+            # we do it is to undo the effect of the previous prepare() by
+            # calling __init__ again and then adding the previously added
+            # objects
+            _added_objects = self._added_objects
+            self.prepared = False
+            Network.__init__(self)
+            for o in _added_objects:
+                self.add(o)
+
     def add(self, *objs):
         """
         Add an object or container of objects to the network
         """
+        self.unprepare()
         for obj in objs:
+            if isinstance(obj, (NeuronGroup, Connection, NetworkOperation)):
+                self._added_objects.append(obj)
             if isinstance(obj, NeuronGroup):
                 if obj not in self.groups:
                     self.groups.append(obj)
@@ -231,6 +249,29 @@ class Network(object):
                     self.add(gco)
             except AttributeError:
                 pass
+            
+    def remove(self, *objs, **kwds):
+        '''
+        Remove an object or sequence of objects from a Network (note that this
+        will force the Network to be prepared from scratch).
+        '''
+        unprepare = kwds.pop('unprepare', True)
+        for obj in objs:
+            if isinstance(obj, (NeuronGroup, Connection, NetworkOperation)):
+                self._added_objects = [o for o in self._added_objects if o is not obj]
+            elif isSequenceType(obj):
+                for o in obj:
+                    self.remove(o, unprepare=False)
+            else:
+                raise TypeError('Only the following types of objects can be removed from a network: NeuronGroup, Connection or NetworkOperation')
+            try:
+                gco = obj.contained_objects
+                if gco is not None:
+                    self.remove(gco, unprepare=False)
+            except AttributeError:
+                pass
+        if unprepare:
+            self.unprepare()
 
     def __call__(self, obj):
         """
@@ -269,6 +310,7 @@ class Network(object):
         + Compresses the connection matrices for faster simulation
         Calling this function is not mandatory but speeds up the simulation.
         '''
+        self.unprepare()
         # Set the clock
         if self.same_clocks():
             self.set_clock()
@@ -295,6 +337,11 @@ class Network(object):
         for C in self.connections:
             C.compress()
 
+        # Compress groups that can be compressed
+        for N in self.groups:
+            if hasattr(N,'compress'):
+                N.compress()
+
         # Experimental support for new propagation code
         if get_global_preference('usenewpropagate') and get_global_preference('useweave'):
             from experimental.new_c_propagate import make_new_connection
@@ -315,6 +362,7 @@ class Network(object):
                           'ops before_connections',
                           'connections',
                           'ops after_connections',
+                          'synapses',
                           'ops before_resets',
                           'resets',
                           'ops after_resets',
@@ -334,6 +382,7 @@ class Network(object):
                           'ops before_connections',
                           'connections',
                           'ops after_connections',
+                          'synapses',
                           'ops end'
                           ]
         self._build_update_schedule()
@@ -354,6 +403,10 @@ class Network(object):
             Calls the 'do_propagate' function of each connection in
             turn, this is typically propagating spikes forward (and
             backward in the case of STDP).
+        'synapses'
+            Calls the 'update' function of each synapse in turn, this
+            may include both updating the state of the synapse as well
+            as altering the state of groups targeted by the synapse.
         'resets'
             Calls the 'reset' function of each group in turn.
         'ops '+name
@@ -408,9 +461,18 @@ class Network(object):
             # we define some simple names for common schedule items
             if isinstance(item, str):
                 if item == 'groups':
-                    objset = self.groups
+                    # we can't directly do something like checking for
+                    # isinstance(group, Synapses) here, as this would lead to
+                    # a circular import
+                    objset = [group for group in self.groups
+                              if not hasattr(group, 'presynaptic')]
                     objfun = 'update'
                     allclocks = False
+                elif item == 'synapses':
+                    objset = [group for group in self.groups
+                              if hasattr(group, 'presynaptic')]
+                    objfun = 'update'
+                    allclocks = False    
                 elif item == 'resets':
                     objset = self.groups
                     objfun = 'reset'
@@ -566,7 +628,16 @@ class Network(object):
         return n
 
     def __repr__(self):
-        return 'Network of' + str(len(self)) + 'neurons'
+        return '<%s of %d groups with a total of %d neurons>' % (self.__class__.__name__,
+                                                                 len(self.groups),
+                                                                 len(self))
+    
+    def __str__(self):
+        return '%s of %d groups with a total of %d neurons' % (self.__class__.__name__,
+                                                               len(self.groups),
+                                                               len(self))
+    
+        
 
     # TODO: obscure custom update schedules might still lead to unpicklable Network object
     def __reduce__(self):
