@@ -13,6 +13,8 @@
 
 import sys, os
 
+on_rtd = os.environ.get('READTHEDOCS', None) == 'True'
+
 building_as = 'html'
 
 # If your extensions are in another directory, add it here. If the directory
@@ -29,6 +31,9 @@ extensions = ['sphinx.ext.autodoc', 'sphinx.ext.ifconfig',
               'sphinx.ext.graphviz', 'sphinx.ext.inheritance_diagram',
               'sphinx.ext.pngmath']
 
+# Inheritance diagram options
+inheritance_graph_attrs = {'rankdir':'TB'}
+
 # Add any paths that contain templates here, relative to this directory.
 templates_path = ['_templates']
 
@@ -46,9 +51,9 @@ copyright = '2008, Romain Brette, Dan Goodman'
 # other places throughout the built documents.
 #
 # The short X.Y version.
-version = '1.3'
+version = '1.4'
 # The full version, including alpha/beta/rc tags.
-release = '1.3.1'
+release = '1.4.1dev'
 
 # There are two options for replacing |today|: either, you set today to some
 # non-false value, then it is used:
@@ -80,7 +85,8 @@ pygments_style = 'sphinx'
 # The style sheet to use for HTML and HTML Help pages. A file of that name
 # must exist either in Sphinx' static/ path, or in one of the custom paths
 # given in html_static_path.
-html_style = 'default.css'
+if not on_rtd:
+    html_style = 'default.css'
 
 html_theme = "default"
 html_theme_options = {
